@@ -20,13 +20,12 @@ if sys.version_info[:2] < (3, 7):
     def nullcontext():
         yield
 
-
 else:
     from contextlib import nullcontext
 
 
 @attr.s
-class SubTestContext(object):
+class SubTestContext:
     msg = attr.ib()
     kwargs = attr.ib()
 
@@ -42,17 +41,17 @@ class SubTestReport(TestReport):
     @property
     def head_line(self):
         _, _, domain = self.location
-        return "{} {}".format(domain, self.sub_test_description())
+        return f"{domain} {self.sub_test_description()}"
 
     def sub_test_description(self):
         parts = []
         if isinstance(self.context.msg, str):
-            parts.append("[{}]".format(self.context.msg))
+            parts.append(f"[{self.context.msg}]")
         if self.context.kwargs:
             params_desc = ", ".join(
-                "{}={!r}".format(k, v) for (k, v) in sorted(self.context.kwargs.items())
+                f"{k}={v!r}" for (k, v) in sorted(self.context.kwargs.items())
             )
-            parts.append("({})".format(params_desc))
+            parts.append(f"({params_desc})")
         return " ".join(parts) or "(<subtest>)"
 
     def _to_json(self):
@@ -80,7 +79,11 @@ def _addSubTest(self, test_case, test, exc_info):
     if exc_info is not None:
         msg = test._message if isinstance(test._message, str) else None
         call_info = make_call_info(
-            ExceptionInfo(exc_info), start=0, stop=0, duration=0, when="call"
+            ExceptionInfo(exc_info, _ispytest=True),
+            start=0,
+            stop=0,
+            duration=0,
+            when="call",
         )
         report = self.ihook.pytest_runtest_makereport(item=self, call=call_info)
         sub_report = SubTestReport._from_test_report(report)
@@ -115,7 +118,7 @@ def subtests(request):
 
 
 @attr.s
-class SubTests(object):
+class SubTests:
     ihook = attr.ib()
     suspend_capture_ctx = attr.ib()
     request = attr.ib()
@@ -130,17 +133,15 @@ class SubTests(object):
 
         # capsys or capfd are active, subtest should not capture
 
-        # pytest<5.4 support: node holds the active fixture
-        capture_fixture_active = getattr(self.request.node, "_capture_fixture", None)
-        if capture_fixture_active is None:
-            # pytest>=5.4 support: capture manager plugin holds the active fixture
-            capman = self.request.config.pluginmanager.getplugin("capturemanager")
-            capture_fixture_active = getattr(capman, "_capture_fixture", None)
+        capman = self.request.config.pluginmanager.getplugin("capturemanager")
+        capture_fixture_active = getattr(capman, "_capture_fixture", None)
 
         if option == "sys" and not capture_fixture_active:
-            fixture = CaptureFixture(SysCapture, self.request)
+            with ignore_pytest_private_warning():
+                fixture = CaptureFixture(SysCapture, self.request)
         elif option == "fd" and not capture_fixture_active:
-            fixture = CaptureFixture(FDCapture, self.request)
+            with ignore_pytest_private_warning():
+                fixture = CaptureFixture(FDCapture, self.request)
         else:
             fixture = None
 
@@ -192,13 +193,28 @@ class SubTests(object):
 
 
 def make_call_info(exc_info, *, start, stop, duration, when):
-    try:
-        return CallInfo(
-            None, exc_info, start=start, stop=stop, duration=duration, when=when
+    return CallInfo(
+        None,
+        exc_info,
+        start=start,
+        stop=stop,
+        duration=duration,
+        when=when,
+        _ispytest=True,
+    )
+
+
+@contextmanager
+def ignore_pytest_private_warning():
+    import warnings
+
+    with warnings.catch_warnings():
+        warnings.filterwarnings(
+            "ignore",
+            "A private pytest class or function was used.",
+            category=pytest.PytestDeprecationWarning,
         )
-    except TypeError:
-        # support for pytest<6: didn't have a duration parameter then
-        return CallInfo(None, exc_info, start=start, stop=stop, when=when)
+        yield
 
 
 @attr.s
@@ -221,3 +237,20 @@ def pytest_report_to_serializable(report):
 def pytest_report_from_serializable(data):
     if data.get("_report_type") == "SubTestReport":
         return SubTestReport._from_json(data)
+
+
+@pytest.hookimpl(tryfirst=True)
+def pytest_report_teststatus(report):
+    if report.when != "call" or not isinstance(report, SubTestReport):
+        return
+
+    if hasattr(report, "wasxfail"):
+        return None
+
+    outcome = report.outcome
+    if report.passed:
+        return outcome, ",", "SUBPASS"
+    elif report.skipped:
+        return outcome, "-", "SUBSKIP"
+    elif outcome == "failed":
+        return outcome, "u", "SUBFAIL"
